/*
 * FxGLUT version 0.12 - GLUT for Voodoo 1 and 2 under Linux
 * Copyright (C) 1999 Christopher John Purnell
 *                    cjp@lost.org.uk
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char *progname;
char line[256];

FILE *in;
FILE *out;

int last = -1;
int size = 0;
int descent = 0;
int encoding = -1;
char *font = "";

int width = 0;
int height = 0;
int xorig = 0;
int yorig = 0;
int xmove = 0;

char *array = 0;

static unsigned char *make_data(int, int);
static void print_header(void);
static void print_data(unsigned char *, int, int);
static void print_footer(char *);

int main(int argc, char **argv)
{
	FILE *fin=0, *fout=0;

	in = stdin;
	out = stdout;

	progname = argv[0];

	if (argc < 2)
	{
		fprintf(stderr, "usage: <name> [<bdffile> [<cfile>]]\n");
		return 1;
	}

	if (argc > 2)
	{
		if ( !(fin = fopen(argv[2], "r")) )
		{
			perror(argv[1]);
			return 1;
		}

		in = fin;

		if (argc > 3)
		{
			if ( !(fout = fopen(argv[3], "w")) )
			{
				perror(argv[1]);
				fclose(fin);
				return 1;
			}
			out = fout;
		}
	}

	print_header();

	while (fgets(line, 256, in))
	{
		char *tok;

		if ( !(tok = strtok(line, " \n")) )
			continue;

		switch(tok[0])
		{
		case 'B':
			if (!strcmp(tok, "BBX"))
			{
				if (! (tok = strtok(0, " \n")))
					break;
				width = atoi(tok);
				if (! (tok = strtok(0, " \n")))
					break;
				height = atoi(tok);
				if (! (tok = strtok(0, " \n")))
					break;
				xorig = -atoi(tok);
				if (! (tok = strtok(0, " \n")))
					break;
				yorig = -atoi(tok);
				break;
			}
			if (!strcmp(tok, "BITMAP"))
			{
				unsigned char *data;
				int w = (width+7)/8, h = height;

				if (! (data = make_data(w, h)) )
					return 1;

				if (encoding >= 0)
				{
					print_data(data, w, h);
				}

				free(data);
				
				break;
			}
			break;
		case 'D':
			if (!strcmp(tok, "DWIDTH"))
			{
				if (! (tok = strtok(0, " \n")) )
					break;
				xmove = atoi(tok);
				break;
			}
			break;
		case 'E':
			if (!strcmp(tok, "ENCODING"))
			{
				if (! (tok = strtok(0, " \n")) )
					break;
				encoding = atoi(tok);
				if (last < encoding) last = encoding;
				break;
			}
			break;
		case 'F':
			if (!strcmp(tok, "FONT"))
			{
				if (! (tok = strtok(0, " \n")) )
					break;
				font = strdup(tok);
			}
			if (!strcmp(tok, "FONTBOUNDINGBOX"))
			{
				if (! (tok = strtok(0, " \n")))
					break;
				if (! (tok = strtok(0, " \n")))
					break;
				size = atoi(tok);
				if (! (tok = strtok(0, " \n")))
					break;
				if (! (tok = strtok(0, " \n")))
					break;
				descent = -atoi(tok);
				break;
			}
			break;
		}
	}

	print_footer(argv[1]);

	if (fout) fclose(fout);
	if (fin) fclose(fin);

	return 0;
}

static unsigned char *make_data(int w, int h)
{
	unsigned char *ptr, buf[3];
	int i, j = h;

	if (! (ptr = malloc(w*h)) )
	{
		perror(progname);
		return 0;
	}

	while (fgets(line, 256, in))
	{
		char *tok = strtok(line, " \n");
		if (!strcmp(tok, "ENDCHAR"))
			break;
		for (i=0,j--; i<w; i++)
		{
			buf[0] = *(tok++);
			buf[1] = *(tok++);
			buf[2] = 0;
			ptr[(j*w)+i] = strtol(buf,0,16);
		}
	}

	return ptr;
}

static void print_header(void)
{
	fprintf(out, "\n/* GENERATED FILE -- DO NOT MODIFY */\n");
	fprintf(out, "\n#include \"bitmap.h\"\n");
}

static void print_data(unsigned char *data, int w, int h)
{
	int i, j, n=w*h;

	fprintf(out, "\n/* char: 0x%x */\n", encoding);

	fprintf(out, "\nstatic const GLubyte ch%udata[] =\n{\n", encoding);

	for (i=0; i<n;)
	{
		fprintf(out, " ");
		for (j=0; j<16 && i<n; j++)
		{
			fprintf(out, "0x%x", data[i++]);
			if (i<n) fprintf(out, ",");
		}
		fprintf(out, "\n");
	}
	fprintf(out, "};\n");

	fprintf(out, "\nstatic const GlutBitmapChar ch%u = {%d,%d,%d,%d,%d,ch%udata};\n",
		encoding, width, height, xorig, yorig, xmove, encoding);


	if (array) array[encoding] = 1;
}

static void print_footer(char *name)
{
	int ch, i;

	fprintf(out, "\nstatic const GlutBitmapChar *chars[] =\n{\n");

	for (ch=0; ch<=last;)
	{
		fprintf(out, " ");
		for (i=0; i<8 && ch<=last; i++,ch++)
		{
			if (array && array[ch])
			{
				fprintf(out, "&ch%u", ch);
			}
			else
			{
				fprintf(out, "0");
			}
			if (ch < last) fprintf(out, ",");
		}
		fprintf(out, "\n");
	}

	fprintf(out, "};\n");

	fprintf(out, "\nconst GlutBitmapFont glutBitmap%s = {\"%s\",%u,%d,%d,chars};\n",
		name, font, last + 1, size, descent);
}
