/*
 * FxGLUT version 0.12 - GLUT for Voodoo 1 and 2 under Linux
 * Copyright (C) 1999 Christopher John Purnell
 *                    cjp@lost.org.uk
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "internal.h"
#include "bitmap.h"

void glutBitmapCharacter(void *font, int c)
{
	const GlutBitmapFont *bfp = (GlutBitmapFont *)font;
	const GlutBitmapChar *bcp;

	if ( ((unsigned)c) >= bfp->num || !(bcp = bfp->table[c]) )
		return;

	glPushClientAttrib(GL_CLIENT_PIXEL_STORE_BIT);

	glPixelStorei(GL_UNPACK_SWAP_BYTES, GL_FALSE);
	glPixelStorei(GL_UNPACK_LSB_FIRST, GL_FALSE);
	glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
	glPixelStorei(GL_UNPACK_SKIP_ROWS, 0);
	glPixelStorei(GL_UNPACK_SKIP_PIXELS, 0);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glBitmap(bcp->width, bcp->height, bcp->xorig, bcp->yorig,
		 bcp->xmove, 0, bcp->bitmap);

	glPopClientAttrib();
}

void glutBitmapString(void *font, const unsigned char *string)
{
	const GlutBitmapFont *bfp = (GlutBitmapFont *)font;
	const GlutBitmapChar *bcp;
	unsigned char c;

	glPushClientAttrib(GL_CLIENT_PIXEL_STORE_BIT);

	glPixelStorei(GL_UNPACK_SWAP_BYTES, GL_FALSE);
	glPixelStorei(GL_UNPACK_LSB_FIRST, GL_FALSE);
	glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
	glPixelStorei(GL_UNPACK_SKIP_ROWS, 0);
	glPixelStorei(GL_UNPACK_SKIP_PIXELS, 0);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	while ( (c = *(string++)) )
	{
		if ( c < bfp->num && (bcp = bfp->table[c]) )
			glBitmap(bcp->width, bcp->height, bcp->xorig,
				 bcp->yorig, bcp->xmove, 0, bcp->bitmap);
	}

	glPopClientAttrib();
}

int glutBitmapWidth(void *font, int c)
{
	const GlutBitmapFont *bfp = (GlutBitmapFont *)font;
	const GlutBitmapChar *bcp;

	if ( ((unsigned)c) >= bfp->num || !(bcp = bfp->table[c]) )
		return 0;
	return bcp->xmove;
}

int glutBitmapLength(void *font, const unsigned char *string)
{
	const GlutBitmapFont *bfp = (GlutBitmapFont *)font;
	const GlutBitmapChar *bcp;
	unsigned char c;
	int length = 0;

	while ( (c = *(string++)) )
	{
		if ( c < bfp->num && (bcp = bfp->table[c]) )
			length += bcp->xmove;
	}

	return length;
}

int glutBitmapHeight(void *font)
{
	const GlutBitmapFont *bfp = (GlutBitmapFont *)font;

	return bfp->height;
}

int glutBitmapDescent(void *font)
{
	const GlutBitmapFont *bfp = (GlutBitmapFont *)font;

	return bfp->descent;
}
