/*
 * FxGLUT version 0.9 - GLUT for Voodoo 1 and 2 under Linux
 * Copyright (C) 1999 Christopher John Purnell
 *                    cjp@lost.org.uk
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <string.h>
#include <stdlib.h>

#include "internal.h"

typedef struct
{
	GrScreenResolution_t res;
	int width, height;
}
glut_resolution;

typedef struct
{
	GrScreenRefresh_t ref;
	int rate;
}
glut_refresh;

static int __glutGameMode = 0;
static int __glutGameRes = 0;
static int __glutGameRef = 0;

#define NUM_RES 6

static glut_resolution __glutResTable[NUM_RES] =
{
	{ GR_RESOLUTION_NONE, -1, -1 },
	{ GR_RESOLUTION_512x384, 512, 384 },
	{ GR_RESOLUTION_640x480, 640, 480 },
	{ GR_RESOLUTION_800x600, 800, 600 },
	{ GR_RESOLUTION_960x720, 960, 720 },
	{ GR_RESOLUTION_1024x768, 1024, 768 }
};

#define NUM_REF 10

static glut_refresh __glutRefTable[NUM_REF] =
{
	{ GR_REFRESH_NONE, -1 },
	{ GR_REFRESH_60Hz, 60 },
	{ GR_REFRESH_70Hz, 70 },
	{ GR_REFRESH_72Hz, 72 },
	{ GR_REFRESH_75Hz, 75 },
	{ GR_REFRESH_80Hz, 80 },
	{ GR_REFRESH_85Hz, 85 },
	{ GR_REFRESH_90Hz, 90 },
	{ GR_REFRESH_100Hz, 100 },
	{ GR_REFRESH_120Hz, 120 }
};

#define WIDTH	0
#define HEIGHT	1
#define DEPTH	2
#define HERTZ	3

#define ISDIGIT(c) (((unsigned char)((c)-'0'))<'\012')

void glutGameModeString(const char *string)
{
	unsigned values[4] = { 640, 480, 16, 60 };
	char c, *str = (char *)string;
	int i;

	while ( (c = *str) )
	{
		unsigned len = strcspn(str, "= \t");
		unsigned val;

		i = -1;

		switch (c)
		{
		case 'b':
			if (len == 3 && !strncmp(str, "bpp", 3))
				i = DEPTH;
			break;
		case 'h':
			if (len == 6 && !strncmp(str, "height", 6))
				i = HEIGHT;
			else if (len == 5 && !strncmp(str, "hertz", 5))
				i = HERTZ;
			break;
		case 'w':
			if (len == 5 && !strncmp(str, "width", 5))
				i = WIDTH;
			break;
		}

		if (i < 0)
		{
			if (ISDIGIT(c))
			{
				if ( (val = strtoul(str, &str, 10)) )
					values[WIDTH] = val;
			}

			if (str[0] == 'x' && ISDIGIT(str[1]))
			{
				if ( (val = strtoul(str+1, &str, 10)) )
					values[HEIGHT] = val;
			}

			if (str[0] == ':' && ISDIGIT(str[1]))
			{
				if ( (val = strtoul(str+1, &str, 10)) )
					values[DEPTH] = val;
			}

			if (str[0] == '@' && ISDIGIT(str[1]))
			{
				if ( (val = strtoul(str+1, &str, 10)) )
					values[HERTZ] = val;
			}
		}
		else
		{
			str += len;

			if (str[0] == '=' && ISDIGIT(str[1]))
			{
				if ( (val = strtoul(str+1, &str, 10)) )
					values[i] = val;
			}
		}

		str += strcspn(str, " \t");
		str += strspn(str, " \t");
	}

	__glutGameRes = 0;

	for (i = 1; i < NUM_RES; i++)
	{
		if (__glutResTable[i].width == values[WIDTH] &&
		    __glutResTable[i].height == values[HEIGHT])
			__glutGameRes = i;
	}

	__glutGameRef = 0;

	for (i = 1; i < NUM_REF; i++)
	{
		if (__glutRefTable[i].rate == values[HERTZ])
			__glutGameRef = i;
	}
}

int glutEnterGameMode(void)
{	
	glutLeaveGameMode();

	if ( (!__glutContext) && __glutGameRes && __glutGameRef )
	{
		__glutGameMode = __glutCreateWindow(__glutResTable[__glutGameRes].res,
						    __glutRefTable[__glutGameRef].ref,
						    __glutResTable[__glutGameRes].width,
						    __glutResTable[__glutGameRes].height);
	}

	return __glutGameMode;
}

void glutLeaveGameMode(void)
{
	if (__glutGameMode)
	{
		if (__glutContext)
			__glutDestroyWindow();
		__glutGameMode = 0;
	}
}

int glutGameModeGet(int mode)
{
	switch (mode)
	{
	case  GLUT_GAME_MODE_ACTIVE:
		return __glutGameMode;
	case GLUT_GAME_MODE_POSSIBLE:
		return (__glutGameRes && __glutGameRef);
	case GLUT_GAME_MODE_WIDTH:
		return __glutResTable[__glutGameRes].width;
	case GLUT_GAME_MODE_HEIGHT:
		return __glutResTable[__glutGameRes].height;
	case GLUT_GAME_MODE_PIXEL_DEPTH:
		return 16;
	case GLUT_GAME_MODE_REFRESH_RATE:
		return __glutRefTable[__glutGameRef].rate;
	default:
		return -1;
	}
}
