/*
 * FxGLUT version 0.9 - GLUT for Voodoo 1 and 2 under Linux
 * Copyright (C) 1999 Christopher John Purnell
 *                    cjp@lost.org.uk
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "internal.h"

#if FXMESA_MAJOR_VERSION >= 3
#define MESA3
#endif

char *__glutProgramName = "GLUT";
int __glutDisplay = -1;
unsigned int __glutDisplayMode = GLUT_RGB | GLUT_SINGLE | GLUT_DEPTH;
int __glutInitX = -1;
int __glutInitY = -1;
int __glutInitWidth = 640;
int __glutInitHeight = 480;

#ifdef MESA3
static int __glutBoard = 0;
#endif

void __glutCloseDisplay(void)
{
	__glutCloseMouse();
	__glutCloseKeyboard();

	if (__glutDisplay != -1)
	{
		__glutDisplay = -1;
		if (__glutContext)
			__glutDestroyWindow();
		fxCloseHardware();
	}
}

void __glutOpenDisplay(void)
{
	static char set = 0;

	if ( (__glutDisplay = fxQueryHardware()) == -1 )
	{
		fprintf(stderr, "%s: No Voodoo Graphics\n", __glutProgramName);
		exit(1);
	}
#ifdef MESA3
	fxMesaSelectCurrentBoard(__glutBoard);
#endif
	if (!set)
	{
		atexit(__glutCloseDisplay);
		set = 1;
	}
	__glutOpenKeyboard();
	__glutOpenMouse();
}

void __glutGetTime(struct timeval *tv)
{
	static char set = 0;
	static struct timeval start;

	if (!set)
	{
		gettimeofday(&start, 0);
		set = 1;
	}
	if (tv)
	{
		*tv = start;
	}
}

void glutInit(int *argcp, char **argv)
{
	char *str;
	int i, j, k;

	if (__glutDisplay != -1)
		return;

	if (argv)
	{
		str = strrchr(argv[0], '/');
		__glutProgramName = str ? str + 1 : argv[0];

		if (argcp)
		{
			k = *argcp;
			for (i = 1; i < k ; i++)
			{
				if ( !strcmp(argv[i], "-width") &&
				     ++i < *argcp )
				{
					__glutInitWidth = atoi(argv[i]);
				}
				else if ( !strcmp(argv[i], "-height") &&
					  ++i < *argcp )
				{
					__glutInitHeight = atoi(argv[i]);
				}
				else if ( !strcmp(argv[i], "-board") &&
					  ++i < *argcp )
				{
#ifdef MESA3
					__glutBoard = atoi(argv[i]);
#endif
				}
				else
				{
					break;
				}
			}

			*argcp = (k -= (i - 1));

			for (j = 1; j < k; j++)
			{
				argv[j] = argv[i++];
			}
			argv[j] = 0;
		}
	}

	__glutOpenDisplay();
	__glutGetTime(0);
}

void glutInitWindowPosition(int x, int y)
{
	__glutInitX = x;
	__glutInitY = y;
}

void glutInitWindowSize(int width, int height)
{
	__glutInitWidth = width;
	__glutInitHeight = height;
}

void glutInitDisplayMode(unsigned int mask)
{
	__glutDisplayMode = mask;
}
