/*
 * FxGLUT version 0.11 - GLUT for Voodoo 1 and 2 under Linux
 * Copyright (C) 1999 Christopher John Purnell
 *                    cjp@lost.org.uk
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <sys/types.h>
#include <asm/bitops.h>
#include <linux/keyboard.h>
#include <linux/kd.h>
#include <linux/vt.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <termios.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>

#include "internal.h"

int __glutFocus = 0;
int __glutKeyFD = -1;
int __glutIgnoreKeyRepeat = 0;

static u_short *keymaps[256];
static u_char shift[NR_SHIFT];
static u_char last_key = -1;
static char clear_shift = 0;
static long lock_state = 0;

static long saved_kbd_mode = K_XLATE;
static struct vt_mode saved_vt_mode;
static struct vt_stat saved_vt_state;
static struct termios saved_termios;

static void (* __glutKeyboardFunc)(unsigned char, int, int);
static void (* __glutKeyboardUpFunc)(unsigned char, int, int);
static void (* __glutSpecialFunc)(int, int, int);
static void (* __glutSpecialUpFunc)(int, int, int);

static struct vt_mode vt_mode =
{
	VT_PROCESS, 0, SIGUSR1, SIGUSR2, 0
};

static void leave_handler(int sig)
{
	if (__glutContext)
	{
#ifdef GLIDE2
		grSstControl(GR_CONTROL_DEACTIVATE);
#endif
#ifdef GLIDE3
		grDisable(GR_PASSTHRU);
#endif
	}
	ioctl(__glutKeyFD, VT_RELDISP, 1);
	__glutFocus = 0;
	__glutPending |= GLUT_NEED_ENTRY;
	clear_shift = 1;
}

static struct sigaction leave_action =
{
	{ leave_handler }, {  }, SA_RESTART, 0
};

static void enter_handler(int sig)
{
	if (__glutContext)
	{
#ifdef GLIDE2
		grSstControl(GR_CONTROL_ACTIVATE);
#endif
#ifdef GLIDE3
		grEnable(GR_PASSTHRU);
#endif
	}
	__glutFocus = 1;
	__glutPending |= GLUT_NEED_ENTRY;
}

static struct sigaction enter_action =
{
	{ enter_handler }, { }, SA_RESTART, 0
};

static void __glutLoadKeymap()
{
	int t, i;
	u_short *map;
	struct kbentry kbent;

	for (t = 0; t < 256; t++)
	{
		kbent.kb_table = t;
		kbent.kb_index = 0;
		
		if (ioctl(__glutKeyFD, KDGKBENT, &kbent) == 0 &&
		    kbent.kb_value != K_NOSUCHMAP &&
		    (map = malloc(128 * sizeof(u_short))))
		{
			map[0] = kbent.kb_value;
			for (i = 1; i < 127; i++)
			{
				kbent.kb_index = i;
				if (ioctl(__glutKeyFD, KDGKBENT, &kbent) == 0)
				{
					map[i] = kbent.kb_value;
				}
			}

			keymaps[t] = map;
		}
		else
		{
			keymaps[t] = 0;
		}
	}
}

static int __glutOpenTTY(int tty)
{
	long mode = KD_GRAPHICS;
	int fd;
	char name[16];

	sprintf(name, "/dev/tty%u", tty);

	if ( (fd = open(name, O_RDWR)) < 0 )
		return -1;

	if (ioctl(fd, KDGETMODE, &mode))
		return -1;

	if (mode != KD_TEXT)
		return -1;

	if (ioctl(fd, KDSETMODE, KD_GRAPHICS))
		return -1;

	ioctl(fd, KDGKBMODE, &saved_kbd_mode);
	ioctl(fd, VT_GETMODE, &saved_vt_mode);
	ioctl(fd, VT_GETSTATE, &saved_vt_state);
	tcgetattr(fd, &saved_termios);

	return fd;
}

void __glutOpenKeyboard(void)
{
	int num;
	struct stat buf;
	struct termios tios;

	if (__glutKeyFD >= 0) return;

	if (!fstat(0, &buf) && 
	    S_ISCHR(buf.st_mode) &&
	    (buf.st_rdev & 0xFF00) == 0x400 &&
	    (num = (buf.st_rdev & 0xFF)))
	{
		__glutKeyFD = __glutOpenTTY(num);
	}

	if ( __glutKeyFD < 0)
	{
		int fd;

		if ( (fd = open("/dev/console", O_RDWR)) >= 0 )
		{
			num = 0;
			ioctl(fd, VT_OPENQRY, &num);
			close(fd);

			if (num)
			{
				setpgid(0,getppid());
				setsid();
				__glutKeyFD = __glutOpenTTY(num);
			}
		}
	}

	if (__glutKeyFD < 0)
	{
		fprintf(stderr, "%s: cannot allocate virtual terminal\n",
			__glutProgramName);
		exit(1);
	}

	tios = saved_termios;
	cfmakeraw(&tios);
	tios.c_oflag |= OPOST;
	tcsetattr(__glutKeyFD, TCSAFLUSH, &tios);

	ioctl(__glutKeyFD, VT_ACTIVATE, num);
	ioctl(__glutKeyFD, VT_WAITACTIVE, num);

	ioctl(__glutKeyFD, KDGKBLED, &lock_state);

	if (ioctl(__glutKeyFD, KDSKBMODE, K_MEDIUMRAW))
	{
		fprintf(stderr, "%s: unable to set keyboard mode\n",
			__glutProgramName);
		exit(1);
	}

	__glutFocus = 1;

	sigaction(SIGUSR1, &leave_action, 0);
	sigaction(SIGUSR2, &enter_action, 0);

	ioctl(__glutKeyFD, VT_SETMODE, &vt_mode);

	__glutLoadKeymap();
}

void __glutCloseKeyboard(void)
{
	if (__glutKeyFD >= 0)
	{
		ioctl(__glutKeyFD, KDSKBMODE, saved_kbd_mode);
		tcsetattr(__glutKeyFD, TCSAFLUSH, &saved_termios);
		ioctl(__glutKeyFD, VT_SETMODE, &saved_vt_mode);
		ioctl(__glutKeyFD, KDSETMODE, KD_TEXT);
		ioctl(__glutKeyFD, VT_ACTIVATE, saved_vt_state.v_active);
		__glutKeyFD = -1;
		__glutFocus = 0;
	}
}

static void __glutDoKeyboard(u_char key, u_char up)
{
	if (up)
	{
		if (__glutKeyboardUpFunc)
			__glutKeyboardUpFunc(key, __glutMouseX, __glutMouseY);
	}
	else
	{
		if (__glutKeyboardFunc)
			__glutKeyboardFunc(key, __glutMouseX, __glutMouseY);
	}
}

static void __glutDoSpecial(int key, u_char up)
{
	if (up)
	{
		if (__glutSpecialUpFunc)
			__glutSpecialUpFunc(key, __glutMouseX, __glutMouseY);
	}
	else
	{
		if (__glutSpecialFunc)
			__glutSpecialFunc(key, __glutMouseX, __glutMouseY);
	}
}

void __glutCheckKeyboard(void)
{
	int i, n = 0;

	if (clear_shift)
	{
		clear_shift = 0;
		for (i=0; i<NR_SHIFT; i++)
			shift[i] = 0;
		last_key = -1;
	}

	if (!__glutFocus)
		return;

	if (ioctl(__glutKeyFD, FIONREAD, &n))
		return;

	while (__glutFocus && n--)
	{
		u_short value, *map;
		u_char c, old, tmp, up;

		if (read(__glutKeyFD, &c, 1) != 1)
			return;

		old = last_key;
		up = c & 0x80;
		c &= 0x7F;

		if (up)
		{
			if (c == old) last_key = old = -1;
		}
		else
		{
			last_key = c;
		}

		tmp = 0;

		for (i = 7; i >= 0; i--)
		{
			tmp <<= 1;
			if (shift[i]) tmp |= 1;
		}

		if ( (shift[KG_CAPSSHIFT] || lock_state & LED_CAP) &&
		     (map = keymaps[0]) && KTYP(map[c]) == KT_LETTER)
		{
			tmp ^= (1 << KG_SHIFT);
		}

		if (!(map = keymaps[tmp]))
			continue;

		value = map[c];

		if (__glutIgnoreKeyRepeat && c == old)
			continue;

		tmp = KVAL(value);
		switch (KTYP(value))
		{
		case KT_META:
//			tmp ^= 0x80;
		case KT_LETTER:
		case KT_LATIN:
			if (tmp == '\177') tmp = '\b';
			__glutDoKeyboard(tmp, up);
			break;
		case KT_FN:
			switch (value)
			{
			case K_FIND:
				__glutDoSpecial(GLUT_KEY_HOME, up);
				break;
			case K_INSERT:
				__glutDoSpecial(GLUT_KEY_INSERT, up);
				break;
			case K_REMOVE:
				__glutDoKeyboard('\177', up);
				break;
			case K_SELECT:
				__glutDoSpecial(GLUT_KEY_END, up);
				break;
			case K_PGUP:
				__glutDoSpecial(GLUT_KEY_PAGE_UP, up);
				break;
			case K_PGDN:
				__glutDoSpecial(GLUT_KEY_PAGE_DOWN, up);
				break;
			default:
				if (tmp < 20)
					__glutDoSpecial(tmp+1, up);
				break;
			}
			break;
		case KT_PAD:
			break;
		case KT_CUR:
			switch (value)
			{
			case K_DOWN:
				__glutDoSpecial(GLUT_KEY_DOWN, up);
				break;
			case K_LEFT:
				__glutDoSpecial(GLUT_KEY_LEFT, up);
				break;
			case K_RIGHT:
				__glutDoSpecial(GLUT_KEY_RIGHT, up);
				break;
			case K_UP:
				__glutDoSpecial(GLUT_KEY_UP, up);
				break;
			}
			break;
		case KT_CONS:
			if (c == old)
				break;
			if (!up)
				ioctl(__glutKeyFD, VT_ACTIVATE, tmp + 1);
			break;
		case KT_SHIFT:
			if (c == old)
				break;
			if (tmp < NR_SHIFT)
			{
				if (up)
				{
					if (shift[tmp]) shift[tmp]--;
				}
				else
				{
					shift[tmp]++;
				}
			}
			break;
		case KT_SPEC:
			switch (value)
			{
			case K_ENTER:
				__glutDoKeyboard('\r', up);
				break;
			case K_SCROLLFORW:
				__glutDoSpecial(GLUT_KEY_PAGE_UP, up);
				break;
			case K_SCROLLBACK:
				__glutDoSpecial(GLUT_KEY_PAGE_DOWN, up);
				break;
			case K_DECRCONSOLE:
				__glutDoSpecial(GLUT_KEY_LEFT, up);
				break;
			case K_INCRCONSOLE:
				__glutDoSpecial(GLUT_KEY_RIGHT, up);
				break;
			}
			if (c == old)
				break;
			switch (value)
			{
			case K_BREAK:
				exit(0);
			case K_CAPS:
				if (!up)
				{
					lock_state ^= LED_CAP;
					ioctl(__glutKeyFD, KDSKBLED, lock_state);
				}
				break;
			case K_NUM:
				if (!up)
				{
					lock_state ^= LED_NUM;
					ioctl(__glutKeyFD, KDSKBLED, lock_state);
				}
				break;
			case K_HOLD:
				if (!up)
				{
					lock_state ^= LED_SCR;
					ioctl(__glutKeyFD, KDSKBLED, lock_state);
				}
				break;
			}
			break;
		}
	}
}

void glutKeyboardFunc(void (*func)(unsigned char, int, int))
{
	__glutKeyboardFunc = func;
}

void glutKeyboardUpFunc(void (*func)(unsigned char, int, int))
{
	__glutKeyboardUpFunc = func;
}

void glutSpecialFunc(void (*func)(int, int, int))
{
	__glutSpecialFunc = func;
}

void glutSpecialUpFunc(void (*func)(int, int, int))
{
	__glutSpecialUpFunc = func;
}

void glutIgnoreKeyRepeat(int ignore)
{
	__glutIgnoreKeyRepeat = ignore;
}

void glutSetKeyRepeat(int repeatMode)
{
}

int glutGetModifiers(void)
{
	int mod = 0;

	if (shift[KG_SHIFT] || shift[KG_SHIFTL] || shift[KG_SHIFTR] ||
	    shift[KG_CAPSSHIFT] || lock_state & LED_CAP)
		mod |= GLUT_ACTIVE_SHIFT;
	if (shift[KG_CTRL] || shift[KG_CTRL] || shift[KG_CTRLR])
		mod |= GLUT_ACTIVE_CTRL;
	if (shift[KG_ALT] || shift[KG_ALTGR])
		mod |= GLUT_ACTIVE_ALT;

	return mod;
}
