/*
 * FxGLUT version 0.9 - GLUT for Voodoo 1 and 2 under Linux
 * Copyright (C) 1999 Christopher John Purnell
 *                    cjp@lost.org.uk
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <stdio.h>

#include "internal.h"

typedef struct glut_timer
{
	struct glut_timer *next;
	struct timeval when; 
	void (*func)(int);
	int value;
}
glut_timer;

unsigned int __glutPending = 0;

void (*__glutIdleFunc)(void) = 0;
void (*__glutEntryFunc)(int) = 0;
void (*__glutVisibilityFunc)(int) = 0;
void (*__glutReshapeFunc)(int, int) = 0;
void (*__glutDisplayFunc)(void) = 0;

static glut_timer *__glutTimerFreeList = 0;
static glut_timer *__glutTimerPending = 0;

void glutMainLoop(void)
{
	while (__glutContext)
	{
		unsigned int pending;
		glut_timer *glutTimer;
		struct timeval now;

		if ((pending = __glutPending))
		{
			__glutPending &= ~pending;

			if (pending & GLUT_NEED_RESHAPE &&
			    __glutReshapeFunc && __glutContext)
			{
				__glutReshapeFunc(__glutWindowWidth,
						  __glutWindowHeight);
			}
			if (pending & GLUT_NEED_DISPLAY &&
			    __glutDisplayFunc && __glutContext)
			{
				__glutDisplayFunc();
			}
			if (pending & GLUT_NEED_ENTRY)
			{
				if (__glutEntryFunc)
					__glutEntryFunc(__glutFocus ?
							GLUT_ENTERED :
							GLUT_LEFT);
				if (__glutVisibilityFunc)
					__glutVisibilityFunc(__glutFocus ?
							GLUT_VISIBLE :
							GLUT_NOT_VISIBLE);
			}
		}

		while ( (glutTimer = __glutTimerPending) )
		{
			gettimeofday(&now, 0);

			if ( (glutTimer->when.tv_sec > now.tv_sec) ||
			     ((glutTimer->when.tv_sec == now.tv_sec) &&
			      (glutTimer->when.tv_usec > now.tv_usec)) )
					break;

			__glutTimerPending = glutTimer->next;
			glutTimer->next = __glutTimerFreeList;
			__glutTimerFreeList = glutTimer;

			glutTimer->func(glutTimer->value);
		}

		if (__glutFocus)
		{
			__glutCheckKeyboard();
			__glutCheckMouse();
		}

		if (!__glutPending)
		{
			if (__glutIdleFunc)
			{
				__glutIdleFunc();
			}
			else
			{
				fd_set rfd;
				int max = 0;
				struct timeval *timeout = 0;

				FD_ZERO(&rfd);
				if (__glutFocus)
				{
					FD_SET(__glutKeyFD, &rfd);
					max = __glutKeyFD + 1;
					if (__glutMouseFD >= 0)
					{
						FD_SET(__glutMouseFD, &rfd);
						if (__glutMouseFD >= max)
							max = __glutMouseFD + 1;
					}
				}
				if ( (glutTimer = __glutTimerPending) )
				{
					gettimeofday(&now, 0);
					now.tv_sec -= glutTimer->when.tv_sec;
					if ( (now.tv_usec -= glutTimer->when.tv_usec) < 0)
					{
						now.tv_usec += 1000000;
						if ( (now.tv_sec -= 1) < 0 )
							continue;
					}
					timeout = &now;
				}
				select(max, &rfd, 0, 0, timeout);
			}
		}
	}
}

void glutTimerFunc(unsigned int millis, void (*func)(int), int value)
{
	glut_timer *new, *gt, **ptr;

	if (!func)
		return;

	if ( (new = __glutTimerFreeList) )
	{
		__glutTimerFreeList = new->next;
	}
	else if (! (new = malloc(sizeof(glut_timer))) )
	{
		fprintf(stderr, "%s: out of memory\n", __glutProgramName);
		exit(1);
	}

	new->func = func;
	new->value = value;

	gettimeofday(&new->when, 0);
	new->when.tv_sec += millis / 1000;
	if ( (new->when.tv_usec += (millis % 1000) * 1000) > 1000000 )
	{
		new->when.tv_usec -= 1000000;
		new->when.tv_sec += 1;
	}

	for (ptr = &__glutTimerPending; (gt = *(ptr)); ptr = &gt->next)
	{
		if ( (gt->when.tv_sec > new->when.tv_sec) ||
		     ((gt->when.tv_sec == new->when.tv_sec) &&
		      (gt->when.tv_usec > new->when.tv_usec)) )
			break;
	}

	*ptr = new;
	new->next = gt;
}
