/*
 * FxGLUT version 0.11 - GLUT for Voodoo 1 and 2 under Linux
 * Copyright (C) 1999 Christopher John Purnell
 *                    cjp@lost.org.uk
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "internal.h"
#include "menu.h"

#define GLUT_MENU_FONT GLUT_BITMAP_HELVETICA_10
#define GLUT_MENU_HEIGHT 13

static GlutMenu *menu_list = 0;
static GlutMenu *current_menu = 0;

static void (* __glutMenuStatusFunc)(int, int, int) = NULL;

GlutMenu *__glutActiveMenu = 0;
GlutMenu *__glutAttachedMenus[3] = { 0, 0, 0 };

static GlutMenu *find_menu(int id)
{
	GlutMenu *menu;

	for (menu = menu_list; menu; menu = menu->next)
	{
		if (menu->id == id)
			break;
	}

	return menu;
}

int glutCreateMenu(void (*func)(int))
{
	GlutMenu *new;

	if (__glutDisplay == -1)
		__glutOpenDisplay();

	if (! (new = malloc(sizeof(GlutMenu))) )
	{
		fprintf(stderr, "%s: out of memory\n", __glutProgramName);
		exit(1);
	}

	new->id = 1;

	if ( (new->next = menu_list) )
	{
		new->id += menu_list->id + 1;
	}

	new->items = 0;
	new->callback = func;
	new->active = 0;
	new->x = 0;
	new->y = 0;
	new->width = 3;
	new->height = 3;

	current_menu = menu_list = new;

	return new->id;
}

static void destroy_menu_item(GlutMenu *menu, GlutMenuItem *item)
{
	GlutMenuItem *itm, **ptr;

	ptr = &menu->items; 
	while ( (itm = *ptr) )
	{
		if (itm == item)
		{
			*ptr = itm->next;
			break;
		}
		ptr = &itm->next;
	}
	
	free(item->label);
	free(item);
}

void glutDestroyMenu(int id)
{
	GlutMenu *menu, *m, **ptr;
	GlutMenuItem *item;
	int i;

	if (! (menu = find_menu(id)) )
		return;

	if (current_menu == menu)
		current_menu = 0;

	if (__glutActiveMenu == menu)
	{
		__glutActiveMenu = 0;
	}

	for (i=0; i<3; i++)
	{
		if (__glutAttachedMenus[i] == menu)
			__glutAttachedMenus[i] = 0;
	}

	for (ptr = &menu_list; (m = *ptr); ptr = &m->next)
	{
		if (m == menu)
		{
			*ptr = menu->next;
		}
		else
		{
			for (item = m->items; item; item = item->next)
			{
				if (item->menu == menu)
					item->menu = 0;
			}
		}
	}

	while ( (item = menu->items) )
	{
		destroy_menu_item(menu, item);
	}

	free(menu);
}

int glutGetMenu(void)
{
	return current_menu ? current_menu->id : 0;
}

void glutSetMenu(int id)
{
	GlutMenu *menu;

	if ( (menu = find_menu(id)) )
	{
		current_menu = menu;
	}
}

static GlutMenuItem *add_menu_item(void)
{
	GlutMenuItem *new, *item, **ptr;

	if (! current_menu)
		return 0;

	if (! (new = malloc(sizeof(GlutMenuItem))) )
	{
		fprintf(stderr, "%s: out of memory\n", __glutProgramName);
		exit(1);
	}

	new->next = 0;
	new->label = 0;
	new->value = 0;
	new->menu = 0;
	new->width = 0;

	ptr = &current_menu->items;
	while ( (item = *ptr) )
	{
		ptr = &item->next;
	}
	*ptr = new;

	return new;
}

static void set_menu_item(GlutMenuItem *item, const char *label,
			  int value, GlutMenu *submenu)
{
	if (! (item->label = strdup(label)) )
	{
		fprintf(stderr, "%s: out of memory\n", __glutProgramName);
		exit(1);
	}

	item->value = value;
	item->menu = submenu;
	item->width = glutBitmapLength(GLUT_MENU_FONT, item->label)
		+ (submenu ? 14: 6);
}

static void glut_recalc_menu(void)
{
	GlutMenuItem *item;

	if (! current_menu)
		return;

	current_menu->width = 3;
	current_menu->height = 3;

	for (item = current_menu->items; (item); item = item->next)
	{
		current_menu->height += GLUT_MENU_HEIGHT;
		if (item->width > current_menu->width)
			current_menu->width = item->width;
	}
}

void glutAddMenuEntry(const char *label, int value)
{
	GlutMenuItem *item;

	if (! (item = add_menu_item()) )
		return;

	set_menu_item(item, label, value, 0);
	glut_recalc_menu();
}

void glutAddSubMenu(const char *label, int submenu)
{
	GlutMenu *menu;
		GlutMenuItem *item;

	if (! (menu = find_menu(submenu)) )
		return;

	if (! (item = add_menu_item()) )
		return;

	set_menu_item(item, label, 0, menu);
	glut_recalc_menu();
}

static GlutMenuItem *find_menu_item(int index)
{
	GlutMenuItem *item;
	int i;

	if (! current_menu)
		return 0;

	for (item = current_menu->items, i = 1;
	     item; item = item->next, i++)
	{
		if (i == index)
			break;
	}

	return item;
}

void glutChangeToMenuEntry(int index, const char *label, int value)
{
	GlutMenuItem *item;

	if (! (item = find_menu_item(index)) )
		return;

	free(item->label);
	set_menu_item(item, label, value, 0);
	glut_recalc_menu();
}

void glutChangeToSubMenu(int index, const char *label, int submenu)
{
	GlutMenuItem *item;
	GlutMenu *menu;

	if (! (item = find_menu_item(index)) )
		return;

	if (! (menu = find_menu(submenu)) )
		return;

	free(item->label);
	set_menu_item(item, label, 0, menu);
	glut_recalc_menu();
}

void glutRemoveMenuItem(int index)
{
	GlutMenuItem *item;

	if (! (item = find_menu_item(index)) )
		return;

	destroy_menu_item(current_menu, item);
	glut_recalc_menu();
}

void glutAttachMenu(int button)
{
	if (current_menu && ((unsigned)button) < 3)
	{
		__glutAttachedMenus[button] = current_menu;
	}
}

void glutDetachMenu(int button)
{
	if (((unsigned)button) < 3)
	{
		if (__glutActiveMenu == __glutAttachedMenus[button])
		{
			__glutActiveMenu = 0;
			__glutPending |= GLUT_NEED_DISPLAY;
		}
		__glutAttachedMenus[button] = 0;
	}
}

static void place_menu(GlutMenu *menu, int x, int y)
{
	int i;

	i = __glutWindowWidth - menu->width;	
	if (x > i) x = i;
	i = __glutWindowHeight - menu->height;
	if (y > i) y = i;

	if (x < 0) x = 0;
	if (y < 0) y = 0;

	menu->x = x;
	menu->y = y;
	menu->active = 0;
}

int __glutMenuButtonDown(GlutMenu *menu)
{
	if (__glutActiveMenu)
		return 0;

	__glutActiveMenu = menu;
	place_menu(menu, __glutMouseX, __glutMouseY);
	__glutSetCursor(GLUT_CURSOR_LEFT_ARROW);

	if (__glutMenuStatusFunc)
		__glutMenuStatusFunc(GLUT_MENU_IN_USE, menu->x, menu->y);

	return 0;
}

static void button_up(GlutMenu *menu)
{
	GlutMenuItem *item;
	GlutMenu *submenu;

	if ( (item = menu->active) )
	{
		if ( (submenu = item->menu) )
		{
			button_up(submenu);
		}
		else
		{
			submenu = current_menu;
			(current_menu = menu)->callback(item->value);
			current_menu = submenu;
		}
	}
}

int __glutMenuButtonUp(GlutMenu *menu)
{
	if (__glutActiveMenu != menu)
		return 0;

	__glutActiveMenu = 0;
	__glutSetCursor(__glutSelectedCursor);

	button_up(menu);

	if (__glutMenuStatusFunc)
		__glutMenuStatusFunc(GLUT_MENU_NOT_IN_USE, menu->x, menu->y);

	return 0;
}

static int check_menu_status(GlutMenu *menu)
{
	GlutMenuItem *item;
	GlutMenu *submenu;
	int x0, x1, y0, y1;

	if ( (item = menu->active) )
	{
		if ( (submenu = item->menu) )
		{
			if (check_menu_status(submenu))
				return 1;
		}
		else
		{
			menu->active = 0;
		}
	}

	x0 = menu->x + 1;
	x1 = menu->x + menu->width - 2;

	if (__glutMouseX <= x0 || __glutMouseX >= x1)
		return 0;

	y0 = menu->y + 1;
	y1 = menu->y + menu->height - 2;

	if (__glutMouseY <= y0 || __glutMouseY >= y1)
		return 0;
	
	for (item = menu->items; item; item = item->next)
	{
		y1 = y0 + GLUT_MENU_HEIGHT;

		if (__glutMouseY > y0 && __glutMouseY < y1)
		{
			if ( (submenu = item->menu) )
			{
				if (menu->active != item)
					place_menu(submenu, x1 - 1, y0 + 1);
				check_menu_status(submenu);
			}
			menu->active = item;
			return 1;
		}

		y0 = y1;
	}

	return 0;
}

static void draw_menu(GlutMenu *menu)
{
	GlutMenuItem *item, *active = menu->active;
	GlutMenu *submenu;
	int x0 = menu->x, x1 = x0 + menu->width;
	int y0 = menu->y, y1 = y0 + menu->height;

	glBegin(GL_QUADS);
	glColor3ub(0, 0, 0);	
	glVertex2i(x0, y0);
	glVertex2i(x0, y1);
	glVertex2i(x1, y1);
	glVertex2i(x1, y0);

	x0 += 1; x1 -= 1;
	y0 += 1; y1 -= 1;

	glColor3ub(127, 127, 127);
	glVertex2i(x0, y0);
	glVertex2i(x0, y1);
	glVertex2i(x1, y1);
	glVertex2i(x1, y0);
	glEnd();

	x0 += 1; x1 -= 1;

	for (item = menu->items; item; item = item->next)
	{
		y1 = y0 + GLUT_MENU_HEIGHT;

		if (menu->active == item)
		{
			glColor3ub(255, 255, 255);
			glBegin(GL_QUADS);
			glVertex2i(x0, y0 + 1);
			glVertex2i(x0, y1);
			glVertex2i(x1, y1);
			glVertex2i(x1, y0 + 1);
			glEnd();
		}

		glColor3ub(0, 0, 0);	
		glRasterPos2i(x0 + 1, y1 - 3);
		glutBitmapString(GLUT_MENU_FONT, item->label);

		if (item->menu)
		{
			glBegin(GL_TRIANGLES);
			glVertex2i(x1 - 6, y0 + 2);
			glVertex2i(x1 - 6, y0 + 12);
			glVertex2i(x1 - 1, y0 + 7);
			glEnd();
		}

		y0 = y1;
	}

	if ( active && (submenu = active->menu) )
	{
		draw_menu(submenu);
	}
}

void __glutDrawMenu(void)
{
	check_menu_status(__glutActiveMenu);
	draw_menu(__glutActiveMenu);
}

void glutMenuStatusFunc(void (*func)(int, int, int))
{
	__glutMenuStatusFunc = func;
}

void glutMenuStateFunc(void (*func)(int))
{
	__glutMenuStatusFunc = (void (*)(int, int, int))func;
}

