/*
 * FxGLUT version 0.9 - GLUT for Voodoo 1 and 2 under Linux
 * Copyright (C) 1999 Christopher John Purnell
 *                    cjp@lost.org.uk
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "internal.h"

fxMesaContext __glutContext;
int __glutWindowWidth = 640;
int __glutWindowHeight = 480;

int glutCreateWindow(const char *title)
{
	if (__glutContext)
		return 0;

	return __glutCreateWindow(GR_RESOLUTION_NONE,GR_REFRESH_NONE,
				  __glutInitWidth, __glutInitHeight);
}

int __glutCreateWindow(GrScreenResolution_t res, GrScreenRefresh_t ref,
		       int width, int height)
{
	int list[32];
	int n = 0;

	if (__glutDisplay == -1)
		__glutOpenDisplay();

	if (__glutDisplayMode & GLUT_DOUBLE)
	{
		list[n++] = FXMESA_DOUBLEBUFFER;
	}
	if (__glutDisplayMode & GLUT_ALPHA)
	{
		list[n++] = FXMESA_ALPHA_SIZE;
		list[n++] = 1;
	}
	if (__glutDisplayMode & GLUT_DEPTH)
	{
		list[n++] = FXMESA_DEPTH_SIZE;
		list[n++] = 1;
	}
	if (__glutDisplayMode & GLUT_STENCIL)
	{
		list[n++] = FXMESA_STENCIL_SIZE;
		list[n++] = 1;
	}
	if (__glutDisplayMode & GLUT_ACCUM)
	{
		list[n++] = FXMESA_ACCUM_SIZE;
		list[n++] = 1;
	}
	list[n] = FXMESA_NONE;

	if (! (__glutContext = ((res == GR_RESOLUTION_NONE) ?
	       fxMesaCreateBestContext(0, width, height, list) :
	       fxMesaCreateContext(0, res, ref, list)) ) )
		return 0;

	fxMesaMakeCurrent(__glutContext);

	__glutPending |= GLUT_NEED_DISPLAY|GLUT_NEED_RESHAPE|GLUT_NEED_ENTRY;

#ifdef GLIDE2
	__glutWindowWidth = grSstScreenWidth();
	__glutWindowHeight = grSstScreenHeight();
#else
	__glutWindowWidth = width;
	__glutWindowHeight = height;
#endif
	return 1;
}

void glutDestroyWindow(int win)
{
	if (win == 1 && __glutContext)
	{
		__glutDestroyWindow();
	}
}

void __glutDestroyWindow(void)
{
	fxMesaContext ctx = __glutContext;

	fxMesaMakeCurrent(__glutContext = 0);
	fxMesaDestroyContext(ctx);
}

void glutSetWindow(int win)
{
	if (win == 1 && __glutContext)
	{
		fxMesaMakeCurrent(__glutContext);
	}
}

int glutGetWindow(void)
{
	return __glutContext ? 1 : 0;
}
